/**
 * @author Eugene Turkov
 * @author <a href="mailto:eugene.turkov@nasa.gov">eugene.turkov@nasa.gov</a>
 * @version: 1.0
 * @since SDK1.5
 * 
 * NASA Ames Research Center Computational Sciences Division Intelligent Data
 * Understanding Group (IDU) Moffett Field, CA 94035
 */

package turkov.lang;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

/**
 * This class is a toolkit for working with strings. Many programs require the
 * use of various string manipulations using regular expressions and iterations -
 * this class is designed to automate many of these cumbersome tasks.
 */
public abstract class TUStringBuilder
{
	/**
	 * Given a time in milliseconds, this method returns a string representation
	 * in Hours, minutes, and seconds.
	 * 
	 * @param milliseconds
	 *            a time in milliseconds.
	 * @return a string such as "5 hours 10 minutes 3 seconds"
	 */
	public static String millisecondsToHoursMinutesSeconds(long milliseconds)
	{
		String result = "";
		int timeInSeconds = (int) milliseconds / 1000;

		int hours;
		int minutes;
		int seconds;

		hours = timeInSeconds / 3600;
		timeInSeconds = timeInSeconds - (hours * 3600);
		minutes = timeInSeconds / 60;
		timeInSeconds = timeInSeconds - (minutes * 60);
		seconds = timeInSeconds;

		if (hours > 0)
		{
			result += String.valueOf(hours) + " hour";
			if (hours > 1)
				result += "s";
			result += " ";
		}
		if (minutes > 0)
		{
			result += String.valueOf(minutes) + " minute";
			if (minutes > 1)
				result += "s";
			result += " ";
		}
		if (seconds > 0)
		{
			result += String.valueOf(seconds) + " second";
			if (seconds > 1)
				result += "s";
		}

		if (result.trim().equals(""))
			result = "0 seconds";

		return result;
	}

	/**
	 * Calling this method loads all of the contents of a given file into a
	 * String object.
	 * 
	 * @param file
	 *            the file to read.
	 * @return a String object containing the contents of the file.
	 */
	public static String generateFromFile(File file) throws IOException
	{
		final String NEW_LINE = System.getProperty("line.separator");
		if (file == null)
			return null;

		if (!file.exists() || !file.isFile())
			throw new FileNotFoundException(file.getAbsolutePath());

		String contents = "";

		String absolutePathToFile = file.getCanonicalPath();

		FileReader fileReader = new FileReader(absolutePathToFile);
		BufferedReader bufferedReader = new BufferedReader(fileReader);

		while (true)
		{
			String line = bufferedReader.readLine();
			if (line == null)
				break;
			else
				contents += line + NEW_LINE;
		}

		if (contents != "")
			contents = contents.substring(0, contents.length() - 1);

		bufferedReader.close();
		fileReader.close();

		return contents;
	}

	/**
	 * Given a File object, this method will generate a String array
	 * representation of this File object if it is indeed a file. Each place
	 * holder within the array will represent a line of the file.
	 * 
	 * @param file
	 *            the File object which is to be converted to String array
	 *            representation.
	 * @return an Array of String where each element is a line of the given
	 *         file.
	 */
	public static String[] generateCollectionFromFile(File file)
			throws IOException
	{
		if (file == null)
			return null;

		ArrayList contents = new ArrayList();

		String absolutePathToFile = file.getCanonicalPath();
		FileReader fileReader = new FileReader(absolutePathToFile);
		BufferedReader bufferedReader = new BufferedReader(fileReader);

		while (true)
		{
			String line = bufferedReader.readLine();
			if (line == null)
				break;
			else
				contents.add(line);
		}

		bufferedReader.close();
		fileReader.close();

		String[] result = new String[contents.size()];
		for (int i = 0; i < result.length; i++)
			result[i] = (String) contents.get(i);

		return result;

	}
}
