/**
 SequenceMiner: multiple anomaly classifier
 
 @author: Eugene Turkov, {eturkov@email.arc.nasa.gov}
 @version: 1.0
 
 NASA Ames Research Center
 Computational Sciences Division
 Intelligent Data Understanding Group (IDU)
 Moffett Field, CA 94035
 */

package turkov.math.algorithms;
import java.util.HashMap;
/**
    Objects of this type should be used for clustering.
 */
public class TUClusterableItem implements Comparable
{    
    /*
     Constructor for an instance of this object.
     
     @param value the data value this item should hold.
     @param id the id of this item. Typically, the id represents the order in
     which this data item was instantiated relative to the other data items.
     */
    public TUClusterableItem(Object value, int id)
    {
            this.id = id;
            this.value = value;
            this.compareMethod = this.COMPARE_BY_ID;
            this.customDataValues = new HashMap();
    }
    
    /*
     This method should be used to retrieve the actual data value of this item.
     
     @return the atual value of this item.
     */
    public Object getValue()
    {
        return this.value;
    }
    
    /*
     Use this method to retrieve the item id of this item.
     
     @return id the id of this data item (typically it is the order in which
     this item was instantiated).
     */
    public int getId()
    {
        return this.id;
    }
    
    public void setId(int id)
    {
        this.id = id;
    }
    
    public void setRank(int rank)
    {
        this.rank = rank;
    }
    
    public int getRank()
    {
        return this.rank;
    }
    
    /*
     This method should be used to compare an instance of this object with other
     objects.
     
     @param object the foreign object to compare to.
     @return -1 if this object is found to be less than the
     foreign object, 0 if this object is equal to the foreign object, and 1 if this
     object is greater than the foreign object.
     */
    public int compareTo(Object object)
    {
        
        final int lessThan = -1;
        final int equalTo = 0;
        final int greaterThan = 1;
        
        if(!(object instanceof TUClusterableItem))
        {
            throw new UnsupportedOperationException("Cannot compare two objects" +
                    " which aren't related. (Both must be comparable on some level).");
        }
        
        else
        {
            TUClusterableItem clusterableItem = (TUClusterableItem)object;
           
            if(this.getCompareMethod().equals(COMPARE_BY_ID))
            {
                if(this.getId() > clusterableItem.getId())
                    return greaterThan;
                else if(this.getId() == clusterableItem.getId())
                    return equalTo;
                else
                    return lessThan;
            }
            
            else
            {
                if(this.getOutlierScore() > clusterableItem.getOutlierScore())
                    return greaterThan;
                else if(this.getOutlierScore() == clusterableItem.getOutlierScore())
                    return equalTo;
                else
                    return lessThan;                
            }
        }                
    }    
    
    /*
     Use this method to set the owner cluster Id of this item (since usually
     items belong to clusters, this method is useful so that a lookup or search
     doesn't have to be performed each time it is neccessary to know which cluster
     this item belongs to if it is being iterated on outside of its owning cluster.
     
     @param ownerClusterId the id of the cluster to which this item belongs.
     */
    public void setOwnerClusterId(int ownerClusterId)
    {
        this.ownerClusterId = ownerClusterId;
    }
    
    /*
     Use this method to get the owner cluster Id of this item (since usually
     items belong to clusters, this method is useful so that a lookup or search
     doesn't have to be performed each time it is neccessary to know which cluster
     this item belongs to if it is being iterated on outside of its owning cluster.
     
     @return ownerClusterId the id of the cluster to which this item belongs.
     */
    public int getOwnerClusterId()
    {
        return ownerClusterId;
    }
    
    public boolean equals(Object object)
    {
        if(object instanceof TUClusterableItem)
        {
            TUClusterableItem item = (TUClusterableItem)object;
            if(item.value.equals(this.value))
                return true;
        }   
        
        return false;
    }    
    
    /*
     This method should be used to set this item's outlier score. The outlier
     score is this item's distance from this its owning cluster's medoid.
     
     @param outlierScore a score which has been computed using some distance
     method (function). The distance between this item and the medoid of this 
     items owning cluster.
     */
    public void setOutlierScore(double outlierScore)
    {
        this.outlierScore = outlierScore;
    }
    
    public String getCompareMethod()
    {
        return this.compareMethod;
    }
    
    public void setCompareMethod(String method)
    {
        this.compareMethod = method;
    }
    
    /*
     This method should be used to get this item's outlier score. The outlier
     score is this item's distance from this its owning cluster's medoid.
     
     @return outlierScore a score which has been computed using some distance
     method (function). The distance between this item and the medoid of this 
     items owning cluster.
     */    
    public double getOutlierScore()
    {
        return this.outlierScore;
    }
    
    public Object getCustomData(Object key)
    {
        return customDataValues.get(key);
    }   
    
    public void setCustomData(Object key, Object value)
    {
        customDataValues.put(key, value);
    }
    
    private Object value;
    private int id;
    private int ownerClusterId;
    private double outlierScore;
    public static final String COMPARE_BY_OUTLIER_SCORE = "COMPARE_BY_OUTLIER_SCORE";
    public static final String COMPARE_BY_ID = "COMPARE_BY_ID";
    private String compareMethod;
    private int rank;
    private HashMap customDataValues;
}
